@@ -1,5 +1,6 @@
 ## function plots reconstructed values for ancestral characters along the edges of the tree
-## written by Liam J. Revell 2012-2017
+## written by Liam J. Revell 2012-2024
+
 contMap<-function(tree,x,res=100,fsize=NULL,ftype=NULL,lwd=4,legend=NULL,
 	lims=NULL,outline=TRUE,sig=3,type="phylogram",direction="rightwards",
 	plot=TRUE,...){
@@ -95,7 +96,7 @@ getState<-function(x,trans){
 
 ## S3 print method for objects of class "contMap"
 ## uses print.densityMap internally
-## written by Liam J. Revell 2012, 2013, 2014, 2015, 2016
+## written by Liam J. Revell 2012, 2013, 2014, 2015, 2016, 2023
 
 plot.contMap<-function(x,...){
 	if(inherits(x,"contMap")){
@@ -107,16 +108,12 @@ plot.contMap<-function(x,...){
 	# get & set optional arguments
 	if(hasArg(legend)) legend<-list(...)$legend
 	else legend<-NULL
+	if(hasArg(sig)) sig<-list(...)$sig
+	else sig<-3
 	if(hasArg(fsize)) fsize<-list(...)$fsize
 	else fsize<-NULL
 	if(hasArg(ftype)) ftype<-list(...)$ftype
 	else ftype<-NULL
-	if(hasArg(outline)) outline<-list(...)$outline
-	else outline<-TRUE
-	if(hasArg(lwd)) lwd<-list(...)$lwd
-	else lwd<-4
-	if(hasArg(sig)) sig<-list(...)$sig
-	else sig<-3
 	if(hasArg(type)) type<-list(...)$type
 	else type<-"phylogram"
 	if(hasArg(mar)) mar<-list(...)$mar
@@ -129,23 +126,139 @@ plot.contMap<-function(x,...){
 	else xlim<-NULL
 	if(hasArg(ylim)) ylim<-list(...)$ylim
 	else ylim<-NULL
-	if(hasArg(hold)) hold<-list(...)$hold
-	else hold<-TRUE
-	if(is.null(legend)) legend<-0.5*max(H)
+	if(hasArg(leg.txt)) leg.txt<-list(...)$leg.txt
+	else leg.txt<-"trait value"
+	if(hasArg(underscore)) underscore<-list(...)$underscore
+	else underscore<-FALSE
+	if(hasArg(outline)) outline<-list(...)$outline
+	else outline<-TRUE
+	if(hasArg(nodes_only)) nodes_only<-list(...)$nodes_only
+	else nodes_only<-FALSE
+	if(hasArg(arc_height)) arc_height<-list(...)$arc_height
+	else arc_height<-2
+	if(is.null(legend)) legend<-if(type=="arc") max(H) else 0.5*max(H)
 	if(is.null(fsize)) fsize<-c(1,1)
 	if(length(fsize)==1) fsize<-rep(fsize,2)
 	if(is.null(ftype)) ftype<-c("i","reg")
 	if(length(ftype)==1) ftype<-c(ftype,"reg")
-	if(hasArg(leg.txt)) leg.txt<-list(...)$leg.txt
-	else leg.txt<-"trait value"
-	# done optional arguments
-	leg.txt<-c(round(lims[1],sig),leg.txt,round(lims[2],sig))
-	plot(x,fsize=fsize,ftype=ftype,lwd=lwd,legend=legend,outline=outline,leg.txt=leg.txt,
-		type=type,mar=mar,direction=direction,offset=offset,xlim=xlim,ylim=ylim,hold=hold)
+	if(nodes_only){
+		if(hasArg(cex)) cex<-list(...)$cex
+		else cex<-c(1.5,1)
+		if(hasArg(lwd)) lwd<-list(...)$lwd
+		else lwd<-c(1,4)
+		obj<-x$tree
+		N<-Ntip(obj)
+		node.cols<-setNames(x$cols[
+			c(names(obj$maps[[which(obj$edge[1,]==(N+1))]][1]),
+			sapply(obj$maps,function(x) names(x[length(x)])))],
+			c(N+1,obj$edge[,2]))
+		node.cols<-node.cols[as.character(1:(N+obj$Nnode))]
+		if(legend&&is.null(ylim)&&type%in%c("phylogram","cladogram")){
+			if(direction%in%c("rightwards","leftwards")) ylim<-c(1-0.12*(N-1),N)
+			else if(direction%in%c("upwards","downwards")) {
+				pp<-par("pin")[2]
+				sw<-(fsize*(max(strwidth(obj$tip.label,units="inches")))+
+					1.37*fsize*strwidth("W",units="inches"))[1]
+				alp<-optimize(function(a,H,sw,pp) (a*1.2*max(H)+sw-pp)^2,H=H,sw=sw,pp=pp,
+					interval=c(0,1e6))$minimum
+				ylim<-if(direction=="downwards") c(min(H)-sw/alp-0.16*max(H),max(H)) else 
+					c(min(H)-0.16*max(H),max(H)+sw/alp)
+			}
+		} else if(is.null(ylim)) ylim<-NULL
+		if(is.null(offset)) {
+			if(type%in%c("cladogram","phylogram")) offset<-0.2*lwd[1]/3+0.2/3
+			else if(type%in%c("fan","arc")) offset<-1
+		}
+		args<-list(...)
+		args$ylim<-ylim
+		args$tree<-as.phylo(obj)
+		args$offset<-offset
+		args$fsize<-fsize[1]
+		args$lwd<-lwd[1]
+		args$arc_height<-arc_height
+		do.call(plotTree,args)
+		pp<-get("last_plot.phylo",envir=.PlotPhyloEnv)
+		xx<-pp$xx
+		yy<-pp$yy
+		if(type%in%c("phylogram","cladogram")){
+			if(direction=="rightwards")
+				xx[1:N]<-xx[1:N]+strwidth(paste(obj$tip.label,"__",sep=""),cex=fsize[1])+offset
+		}
+		if(type=="arc"){ 
+			DROP<-Ntip(x$tree)+1
+			points(xx[-DROP],yy[-DROP],pch=if(outline) 21 else 16,
+				col=if(outline) par()$fg else node.cols[-DROP],
+				bg=if(outline) node.cols[-DROP] else NULL,
+				cex=c(rep(cex[2],N),rep(cex[1],obj$Nnode-length(DROP))))
+		} else {
+			points(xx,yy,pch=if(outline) 21 else 16,
+				col=if(outline) par()$fg else node.cols,
+				bg=if(outline) node.cols else NULL,
+				cex=c(rep(cex[2],N),rep(cex[1],obj$Nnode)))
+		}
+		if(legend){
+			if(is.logical(legend)) legend<-0.5*max(H)
+			if(length(leg.txt)==1) 
+				leg.txt<-c(round(lims[1],sig),leg.txt,round(lims[2],sig))
+			if(type%in%c("phylogram","cladogram")){
+				if(direction%in%c("rightwards","leftwards")){
+					add.color.bar(legend,x$cols,title=leg.txt[2],
+						as.numeric(leg.txt[c(1,3)]),
+						digits=sig,prompt=FALSE,
+						x=if(direction=="leftwards") max(H)-legend else 0,
+						y=1-0.08*(N-1),lwd=lwd[2],
+						fsize=fsize[2],outline=outline,
+						direction=if(!is.null(xlim)) 
+						if(xlim[2]<xlim[1]) "leftwards" else 
+						"rightwards" else "rightwards")
+				} else if(direction%in%c("upwards","downwards")){
+					add.color.bar(legend,x$cols,title=leg.txt[2],
+						as.numeric(leg.txt[c(1,3)]),
+						digits=sig,prompt=FALSE,x=1,
+						y=ylim[1]+0.04*max(nodeHeights(x$tree)),
+						lwd=lwd[2],outline=outline,
+						fsize=fsize[2],direction="rightwards",
+						subtitle=paste("length=",round(legend,3),
+						sep=""))
+				}
+			} else if(type=="arc"){
+				add.color.bar(legend,x$cols,
+					title=leg.txt[2],
+					as.numeric(leg.txt[c(1,3)]),
+					digits=sig,prompt=FALSE,
+					outline=outline,
+					x=mean(par()$usr[1:2])-0.5*legend,
+					y=par()$usr[3]+0.1*diff(par()$usr[3:4]),
+					lwd=lwd[2],
+					fsize=fsize[2])
+			} else if(type=="fan"){
+				add.color.bar(legend,x$cols,
+					title=leg.txt[2],
+					as.numeric(leg.txt[c(1,3)]),
+					digits=sig,prompt=FALSE,
+					outline=outline,
+					x=0.9*par()$usr[1],
+					y=0.9*par()$usr[3],
+					lwd=lwd[2],
+					fsize=fsize[2])
+			}
+		}
+	} else {
+		if(hasArg(lwd)) lwd<-list(...)$lwd
+		else lwd<-4
+		if(hasArg(hold)) hold<-list(...)$hold
+		else hold<-TRUE
+		# done optional arguments
+		leg.txt<-c(round(lims[1],sig),leg.txt,round(lims[2],sig))
+		plot(x,fsize=fsize,ftype=ftype,lwd=lwd,legend=legend,outline=outline,leg.txt=leg.txt,
+			type=type,mar=mar,direction=direction,offset=offset,xlim=xlim,ylim=ylim,hold=hold,
+			underscore=underscore,arc_height=arc_height)
+	}
 }
 
 ## S3 print method for object of class 'contMap'
 ## written by Liam J. Revell 2013
+
 print.contMap<-function(x,digits=6,...){
 	cat("Object of class \"contMap\" containing:\n\n")
 	cat(paste("(1) A phylogenetic tree with ",length(x$tree$tip.label)," tips and ",x$tree$Nnode," internal nodes.\n\n",sep=""))
@@ -153,17 +266,19 @@ print.contMap<-function(x,digits=6,...){
 }
 
 ## drop tips from an object of class 'contMap'
-## written by Liam J. Revell 2014
-drop.tip.contMap<-function(x,tip,...){
-	if(!inherits(x,"contMap")) cat("x should be an object of class \"contMap\"\n")
+## written by Liam J. Revell 2014, 2023
+
+drop.tip.contMap<-function(phy,tip,...){
+	if(!inherits(phy,"contMap")) cat("phy should be an object of class \"contMap\"\n")
 	else {
-		x$tree<-drop.tip.simmap(x$tree,tip,...)
-		return(x)
+		phy$tree<-drop.tip.simmap(phy$tree,tip,...)
+		return(phy)
 	}
 }
 
 ## add error bars to contMap plot
 ## written by Liam J. Revell 2017
+
 errorbar.contMap<-function(obj,...){
 	if(hasArg(x)) x<-list(...)$x
 	else x<-setNames(sapply(1:Ntip(obj$tree),function(x,obj){
@@ -206,7 +321,7 @@ errorbar.contMap<-function(obj,...){
 	}
 }
 
-keep.tip.contMap<-function(x,tip,...){
-	tips<-setdiff(x$tree$tip.label,tip)
-	drop.tip.contMap(x,tips,...)
+keep.tip.contMap<-function(phy,tip,...){
+	tips<-setdiff(phy$tree$tip.label,tip)
+	drop.tip.contMap(phy,tip=tips,...)
 }
